﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=7
@EndOfDesignText@
'Static code module
Sub Process_Globals
	Public BANano As BANano
	Public const DB_BOOL As String = "BOOL"
	Public const DB_INT As String = "INT"
	Public const DB_STRING As String = "STRING"
	Public const DB_DATE As String = "DATE"
	Public const DB_CREATE As String = "CREATE"
	Public const DB_INSERT As String = "INSERT"
	Public const DB_UPDATE As String = "UPDATE"
	Public const DB_UPDATEALL As String = "UPDATEALL"
	Public const DB_SELECT As String = "SELECT"
	Public const DB_EXISTS As String = "EXISTS"
	Public const DB_CREATEINDEX As String = "CREATEINDEX"
	Public const DB_DELETE As String = "DELETE"
	Public const DB_DROPTABLE As String = "DROPTABLE"
	Public const DB_DROPDATABASE As String = "DROPDATABASE"
	Public const DB_HTML As String = "HTML"
	Public const DB_ATTACH As String = "ATTACH"
	Public const DB_XLS As String = "XLS"
End Sub


Sub Export2XLS(tblName As String) As String
	Dim sql As String = $"SELECT * INTO XLSX("${tblName}") FROM ${EscapeField(tblName)}"$
	Return sql
End Sub

Sub AttachJSON(dbName As String, jsonFile As String) As String
	Dim sql As String = $"ATTACH FILESTORAGE DATABASE ${dbName}("${jsonFile}");"$
	Return sql
End Sub

Sub AttachSQLite(dbName As String, sqliteFile As String) As String
	Dim sql As String = $"['ATTACH SQLITE DATABASE ${dbName}("${sqliteFile}")','USE ${dbName}']"$
	Return sql
End Sub

Sub CreateHTMLTable(elID As String, tblName As String) As String
	elID = elID.tolowercase
	Dim sql As String = $"SELECT * INTO HTML("#${elID}",{headers:true}) FROM ${tblName}"$
	Return sql
End Sub

'drop a table
Sub DropTable(tblName As String) As String
	Dim sql As String = $"DROP TABLE ${EscapeField(tblName)}"$
	Return sql
End Sub

'drop a database
Sub DropDatabase(dbName As String) As String
	Dim sql As String = $"DROP INDEXEDDB DATABASE ${EscapeField(dbName)}"$
	Return sql
End Sub

'the map should be the fieldname and unique parameter
'do not include the primary key, unique not implemented
Sub CreateIndexes(tblName As String, idxMap As Map) As List
	Dim commands As List
	commands.Initialize 
	If idxMap.Size = 0 Then
		Log($"CreateIndexes: ${tblName} - idxMap map empty!"$)
		Return
	End If
	Dim iTot As Int = idxMap.Size - 1
	Dim iCnt As Int
	For iCnt = 0 To iTot
		Dim idxName As String = idxMap.GetKeyAt(iCnt)
		Dim idxUnique As Boolean = idxMap.Get(idxName)
		Dim idxKey As String = $"${tblName}_${idxName}"$
		Dim qry As String = $"CREATE INDEX ${EscapeField(idxKey)} ON ${EscapeField(tblName)}(${EscapeField(idxName)})"$
		commands.Add(qry)
	Next
	Return commands
End Sub

Private Sub EscapeField(f As String) As String
	Return $"[${f}]"$
End Sub

'return a sql command to create the table
public Sub CreateTableSQL(tblName As String, tblFields As Map, PK As String) As String
	If tblFields.Size = 0 Then
		Log($"CreateTableSQL: ${tblName} - tblFields map empty!"$)
		Return Null
	End If
	Dim fldName As String
	Dim fldType As String
	Dim fldTot As Int
	Dim fldCnt As Int
	fldTot = tblFields.Size - 1
	Dim sb As StringBuilder
	sb.Initialize
	sb.Append("(")
	For fldCnt = 0 To fldTot
		fldName = tblFields.GetKeyAt(fldCnt)
		fldType = tblFields.Get(fldName)
		If fldCnt > 0 Then
			sb.Append(", ")
		End If
		sb.Append(EscapeField(fldName))
		sb.Append(" ")
		sb.Append(fldType)
		If fldName.EqualsIgnoreCase(PK) Then
			sb.Append(" PRIMARY KEY")
		End If
	Next
	sb.Append(")")
	'define the qry to execute
	Dim query As String = "CREATE TABLE IF NOT EXISTS " & EscapeField(tblName) & " " & sb.ToString
	Return query
End Sub

'return a sql map to insert record to table. sql = query string, values = list of values
Sub InsertSQLMap(tblName As String, tblFields As Map) As Map
	If tblFields.Size = 0 Then
		Log($"InsertSQLMap: ${tblName} - tblFields map empty!"$)
		Return Null
	End If
	Dim sb As StringBuilder
	Dim columns As StringBuilder
	Dim values As StringBuilder
	Dim listOfValues As List
	Dim iCnt As Int
	Dim iTot As Int
	sb.Initialize
	columns.Initialize
	values.Initialize
	listOfValues.Initialize
	sb.Append($"INSERT INTO ${EscapeField(tblName)} ("$)
	iTot = tblFields.Size - 1
	For iCnt = 0 To iTot
		Dim col As String = tblFields.GetKeyAt(iCnt)
		Dim value As String = tblFields.Get(col)
		If iCnt > 0 Then
			columns.Append(", ")
			values.Append(", ")
		End If
		columns.Append(EscapeField(col))
		values.Append("?")
		listOfValues.Add(value)
	Next
	sb.Append(columns.ToString)
	sb.Append(") VALUES (")
	sb.Append(values.ToString)
	sb.Append(")")
	Dim m As Map
	m.Initialize
	m.Put("sql", sb.ToString)
	m.Put("values", listOfValues)
	Return m
End Sub

'return a sql to update record of table where one exists
Sub UpdateWhereSQLMap(tblName As String, tblFields As Map, tblWhere As Map) As Map
	If tblWhere.Size = 0 Then
		Log($"UpdateWhereSQLMap: ${tblName} - tblWhere map empty!"$)
		Return Null
	End If
	If tblFields.Size = 0 Then
		Log($"UpdateWhereSQLMap: ${tblName} - tblFields map empty!"$)
		Return Null
	End If
	Dim sb As StringBuilder
	Dim args As List
	sb.Initialize
	args.Initialize
	sb.Append($"UPDATE ${EscapeField(tblName)} SET "$)
	Dim i As Int
	Dim iTot As Int = tblFields.Size - 1
	For i = 0 To iTot
		Dim col As String = tblFields.GetKeyAt(i)
		Dim colvalue As String = tblFields.Get(col)
		sb.Append(col)
		If i <> iTot Then
			sb.Append("=?,")
		Else
			sb.Append("=?")
		End If
		args.Add(colvalue)
	Next
	sb.Append(" WHERE ")
	Dim iWhere As Int = tblWhere.Size - 1
	For i = 0 To iWhere
		If i > 0 Then
			sb.Append(" AND ")
		End If
		Dim colwhere As String = tblWhere.GetKeyAt(i)
		Dim colvalue As String = tblWhere.Get(colwhere)
		sb.Append(colwhere)
		sb.Append(" = ?")
		args.Add(colvalue)
	Next
	Dim m As Map
	m.Initialize
	m.Put("sql", sb.tostring)
	m.Put("values", args)
	Return m
End Sub

'return a sql to delete record of table where one exists
Sub DeleteWhereSQLMap(tblName As String, tblWhere As Map) As Map
	If tblWhere.Size = 0 Then
		Log($"DeleteWhereSQLMap: ${tblName} - tblWhere map empty!"$)
		Return Null
	End If
	Dim sb As StringBuilder
	Dim args As List
	sb.Initialize
	args.Initialize
	sb.Append($"DELETE FROM ${EscapeField(tblName)} WHERE "$)
	Dim i As Int
	Dim iWhere As Int = tblWhere.Size - 1
	For i = 0 To iWhere
		If i > 0 Then
			sb.Append(" AND ")
		End If
		Dim colwhere As String = tblWhere.GetKeyAt(i)
		Dim colvalue As String = tblWhere.Get(colwhere)
		sb.Append(colwhere)
		sb.Append(" ? ")
		args.Add(colvalue)
	Next
	Dim m As Map
	m.Initialize
	m.Put("sql", sb.tostring)
	m.Put("values", args)
	Return m
End Sub

'return a sql to update all records of table
Sub UpdateSQLMap(tblName As String, tblFields As Map) As Map
	If tblFields.Size = 0 Then
		Log($"UpdateSQLMap: ${tblName} - tblFields map empty!"$)
		Return Null
	End If
	Dim sb As StringBuilder
	Dim args As List
	sb.Initialize
	args.Initialize
	sb.Append($"UPDATE ${EscapeField(tblName)} SET "$)
	Dim i As Int
	Dim iTot As Int = tblFields.Size - 1
	For i = 0 To iTot
		Dim col As String = tblFields.GetKeyAt(i)
		Dim colvalue As String = tblFields.Get(col)
		sb.Append(col)
		If i <> iTot Then
			sb.Append("=?,")
		Else
			sb.Append("=?")
		End If
		args.Add(colvalue)
	Next
	Dim m As Map
	m.Initialize
	m.Put("sql", sb.tostring)
	m.Put("values", args)
	Return m
End Sub

'return a sql to select record of table where one exists
Sub SelectWhereSQLMap(tblName As String, tblWhere As Map) As Map
	If tblWhere.Size = 0 Then
		Log($"SelectWhereSQLMap: ${tblName} - tblWhere map empty!"$)
		Return Null
	End If
	Dim sb As StringBuilder
	Dim args As List
	sb.Initialize
	args.Initialize
	sb.Append($"SELECT * FROM ${EscapeField(tblName)} WHERE "$)
	Dim i As Int
	Dim iWhere As Int = tblWhere.Size - 1
	For i = 0 To iWhere
		If i > 0 Then
			sb.Append(" AND ")
		End If
		Dim colwhere As String = tblWhere.GetKeyAt(i)
		Dim colvalue As String = tblWhere.Get(colwhere)
		sb.Append(colwhere)
		sb.Append(" = ?")
		args.Add(colvalue)
	Next
	Dim m As Map
	m.Initialize
	m.Put("sql", sb.tostring)
	m.Put("values", args)
	Return m
End Sub
